import { Context, DynamoDBStreamEvent } from 'aws-lambda';
import log from '@dazn/lambda-powertools-logger';
import Client from '@svc/lib/models/Client';
import { Marshaller } from '@aws/dynamodb-auto-marshaller';
import { LowLevelClientProfile } from '@improovy/types';
import { addSeenDDBRecord, wasDDBRecordSeen } from '@svc/lib/ddbRepos/idempotency-repo';

const marshaller = new Marshaller();

export const handler = async (event: DynamoDBStreamEvent, ctx: Context) => {
  log.debug('Received event', { event });
  
  for (let record of event.Records) {
    const recordWasSeen = await wasDDBRecordSeen(record, ctx.functionName);
    if (recordWasSeen) continue;

    const client = marshaller.unmarshallItem(record.dynamodb!.NewImage!) as unknown as LowLevelClientProfile;
    await addGclidToProjects(client);

    await addSeenDDBRecord(record, ctx.functionName);
  }
  return;
};

const addGclidToProjects = async (clientInterface: LowLevelClientProfile) => {
  const client = new Client(clientInterface);
  const projects = await client.fetchProjects();
  if (!projects) return;
  const promises = projects.map(project => {
    if (project.gclid) return Promise.resolve();
    
    return project.updateGclid(client.gclid);
  });

  return Promise.all(promises);
};