import { Context, DynamoDBStreamEvent } from 'aws-lambda';
import log from '@dazn/lambda-powertools-logger';
import Client from '@svc/lib/models/Client';
import { Marshaller } from '@aws/dynamodb-auto-marshaller';
import { addSeenDDBRecord, wasDDBRecordSeen } from '@svc/lib/ddbRepos/idempotency-repo';
import { LowLevelCallRail } from '@improovy/types';
import { assembleCallRail } from '@svc/lib/ddbRepos/callRail-repo';
import CallRail from '@svc/lib/models/CallRail';
import { ChangeSource } from '@improovy/types';

const marshaller = new Marshaller();

export const handler = async (event: DynamoDBStreamEvent, ctx: Context) => {
  log.debug('Received event', { event });
  
  for (let record of event.Records) {
    const recordWasSeen = await wasDDBRecordSeen(record, ctx.functionName);
    if (recordWasSeen) continue;

    const lowLevelCallRail = marshaller.unmarshallItem(record.dynamodb!.NewImage!) as unknown as LowLevelCallRail;
    const callRailInterface = assembleCallRail(lowLevelCallRail);
    const callRail = new CallRail(callRailInterface);
    log.debug('phoneNumber', {phoneNumber: callRail.phoneNumber})
    const client = await Client.fetchByPhoneNumber(callRail.phoneNumber);

    log.debug('client', { client })
    if (client) {
      client.changeSource = ChangeSource.aws;
      await client.updateGclid(callRail.gclid);
      callRail.changeSource = ChangeSource.aws
      await callRail.updateIsAttributed(true)
    }

    await addSeenDDBRecord(record, ctx.functionName);


  }
  return;
};