import { Context, DynamoDBStreamEvent } from 'aws-lambda';
import log from '@dazn/lambda-powertools-logger';
import Client from '@svc/lib/models/Client';
import { Marshaller } from '@aws/dynamodb-auto-marshaller';
import { ContactType, LowLevelClientProfile } from '@improovy/types';
import { addSeenDDBRecord, wasDDBRecordSeen } from '@svc/lib/ddbRepos/idempotency-repo';
import CallRail from '@svc/lib/models/CallRail';
import { ChangeSource } from '@improovy/types';

const marshaller = new Marshaller();

export const handler = async (event: DynamoDBStreamEvent, ctx: Context) => {
  log.debug('Received event', { event });
  
  for (let record of event.Records) {
    const recordWasSeen = await wasDDBRecordSeen(record, ctx.functionName);
    if (recordWasSeen) continue;

    const LowLevelClientProfile = marshaller.unmarshallItem(record.dynamodb!.NewImage!) as unknown as LowLevelClientProfile;
    
    const client = await Client.fetch(LowLevelClientProfile.uuid)

    if (!client) throw new Error("received client not found, probably transient error?");

    const foundCallRails = [];
    
    for (let contactMethod of client.contactMethods) {
      if (contactMethod.contactType !== ContactType.PHONE) continue;

      const callRail = await CallRail.fetch(contactMethod.contactAddress);
      if (callRail) foundCallRails.push(callRail);
    }

    if (!foundCallRails.length) return;

    if (foundCallRails.length > 1) throw new Error("found more than one callrail record");

    const callRail = foundCallRails[0];

    client.changeSource = ChangeSource.aws;
    await client.updateGclid(callRail.gclid);
    callRail.changeSource = ChangeSource.aws
    await callRail.updateIsAttributed(true)

    await addSeenDDBRecord(record, ctx.functionName);
  }

  return;
};