import { Context, DynamoDBStreamEvent } from 'aws-lambda';
import log from '@dazn/lambda-powertools-logger';
import { Marshaller } from '@aws/dynamodb-auto-marshaller';
import { LowLevelProjectDetails } from '@improovy/types';
import Project from '@svc/lib/models/Project';
import { addSeenDDBRecord, wasDDBRecordSeen } from '@svc/lib/ddbRepos/idempotency-repo';

const marshaller = new Marshaller();

export const handler = async (event: DynamoDBStreamEvent, ctx: Context) => {
  log.debug('Received event', { event });
  
  const promises = [];
  for (let record of event.Records) {
    const recordWasSeen = await wasDDBRecordSeen(record, ctx.functionName);
    if (recordWasSeen) continue;

    const project = marshaller.unmarshallItem(record.dynamodb!.NewImage!) as unknown as LowLevelProjectDetails;
    await checkClientGclid(project);
    
    await addSeenDDBRecord(record, ctx.functionName);
  }
  return;
};

const checkClientGclid = async (projectInterface: LowLevelProjectDetails) => {
  const project = new Project(projectInterface);
  const client = await project.fetchClient();

  // if client is null, it's not in the database. Another lambda function will handle adding it to the database,
  // at which point another lambda function will add the gclid to all of the client's projects
  if (!client || !client.gclid) return Promise.resolve();
  
  return project.updateGclid(client.gclid);
};