import { Context, DynamoDBStreamEvent } from 'aws-lambda';
import log from '@dazn/lambda-powertools-logger';
import { Marshaller } from '@aws/dynamodb-auto-marshaller';
import { addSeenDDBRecord, wasDDBRecordSeen } from '@svc/lib/ddbRepos/idempotency-repo';
import { LowLevelGAConversion } from '@improovy/types';
import googleAdsAPI from '@svc/lib/googleAdsAPI';

const marshaller = new Marshaller({unwrapNumbers: true});

export const handler = async (event: DynamoDBStreamEvent, ctx: Context) => {
  log.debug('Received event', { event });
  
  for (let record of event.Records) {
    const recordWasSeen = await wasDDBRecordSeen(record, ctx.functionName);
    if (recordWasSeen) continue;

    const lowLevelConversion = marshaller.unmarshallItem(record.dynamodb!.NewImage!) as unknown as LowLevelGAConversion;
     
    if (!lowLevelConversion.isTest) {
      await googleAdsAPI.uploadConversion(lowLevelConversion)
    }

    await addSeenDDBRecord(record, ctx.functionName);


  }
  return;
};