import { Context, DynamoDBStreamEvent } from 'aws-lambda';
import log from '@dazn/lambda-powertools-logger';
import { Marshaller } from '@aws/dynamodb-auto-marshaller';
import { addSeenDDBRecord, wasDDBRecordSeen } from '@svc/lib/ddbRepos/idempotency-repo';
import GAConversion from '@svc/lib/models/GAConversion';
import { LowLevelProjectDetails } from '@improovy/types';

const marshaller = new Marshaller();

export const handler = async (event: DynamoDBStreamEvent, ctx: Context) => {
  log.debug('Received event', { event });
  
  for (let record of event.Records) {
    const recordWasSeen = await wasDDBRecordSeen(record, ctx.functionName);
    if (recordWasSeen) continue;
    
    const newLowLevelProject = marshaller.unmarshallItem(record.dynamodb!.NewImage!) as unknown as LowLevelProjectDetails;
    const oldLowLevelProject = marshaller.unmarshallItem(record.dynamodb!.OldImage!) as unknown as LowLevelProjectDetails;

    let conversions;

    if (newLowLevelProject.gclid) {
      conversions = await GAConversion.fetchByProject(newLowLevelProject)
    } else {
      conversions = await GAConversion.fetchByProject(oldLowLevelProject);
    }

    if (!conversions) throw new Error("conversions not found");
    
    for(let conversion of conversions) {
      await conversion.hardDelete();
    };

    await addSeenDDBRecord(record, ctx.functionName);
  }
  return;
};