import { getTableName } from '@improovy/utils';
import { AWS_REGION, ddb, lambdaConfig } from '@svc/config';
import { buildLowLevelGAConversion } from '@svc/lib/ddbRepos/gaConversions-repo';
import TestDataGenerator from '@tests/utils/testDataGenerator';

const testDataGenerator = new TestDataGenerator();

const lambdaFunctionName = `${lambdaConfig.functionNamePrefix}DDBFireGAConversion`;

const lambdaLogs = {
  region: AWS_REGION,
  function: lambdaFunctionName,
};

const gaConversionTable = {
  region: AWS_REGION,
  table: getTableName('GA_CONVERSIONS')
}

describe('`fireGAConversion` Lambda function', () => {
  const testConversion = testDataGenerator.getGAConversion();
  const lowLevelConversion = buildLowLevelGAConversion(testConversion)

  it.concurrent('is triggered when a conversion is deleted', async () => {
    await testConversion.put();
    await testConversion.hardDelete();
    const expectedLog = testConversion.gclid;
    await expect({
      ...lambdaLogs,
      timeout: 50000,
    }).toHaveLog(expectedLog);
  });

  afterAll(async () => testDataGenerator.deleteAll());
});