import { Marshaller } from '@aws/dynamodb-auto-marshaller';
import { lambdaConfig } from '@svc/config';
import { handler } from '@svc/handlers/fireGAConversion';
import { buildLowLevelGAConversion } from '@svc/lib/ddbRepos/gaConversions-repo';
import googleAdsAPI from '@svc/lib/googleAdsAPI';
import { GAConversionType } from '@improovy/types';
import uuid from '@svc/lib/uuid';
import TestDataGenerator from '@tests/utils/testDataGenerator';
import { Context, DynamoDBStreamEvent } from 'aws-lambda';

jest.mock('@svc/lib/googleAdsAPI', () => {
  return {
    __esModule: true,
    default: {
      uploadConversion: jest.fn(() => Promise.resolve())
    }
  };
});

jest.mock('@svc/lib/ddbRepos/idempotency-repo', () => {
  return {
    __esModule: true,
    wasDDBRecordSeen: () => false,
    addSeenDDBRecord: () => Promise.resolve(),
  };
});

const marshaller = new Marshaller();

const testDataGenerator = new TestDataGenerator(false);

const lambdaFunctionName = `${lambdaConfig.functionNamePrefix}DDBFireGAConversion`;

describe('`fireGAConversion` Lambda function', () => {
   
  it('invokes the google ads api instance with the conversion', async () => {
    const testConversion = testDataGenerator.getGAConversion({fieldsToExclude: ['isTest'], overrides: { conversionType: GAConversionType.QUOTE_RECEIVED, conversionValue: 1000}});
    const lowLevelTestConversion = buildLowLevelGAConversion(testConversion)

    const event: DynamoDBStreamEvent = {
      Records: [{
        eventID: uuid(),
        dynamodb: {
          // @ts-expect-error
          NewImage: marshaller.marshallItem(lowLevelTestConversion)
        }
      }]
    }

    await handler(event, {functionName: lambdaFunctionName} as Context)

    expect(googleAdsAPI.uploadConversion).toHaveBeenCalledTimes(1)
    expect(googleAdsAPI.uploadConversion).toHaveBeenCalledWith(lowLevelTestConversion)
  });
});