import uuid from '@svc/lib/uuid';
import { generateValidatedUuid, isValidUuid } from '@svc/lib/models/modelUtils';

const badUuid = 'PDID1234';
const goodUuid = 'pdid1234';

jest.mock('@svc/lib/uuid', () => {
  const mockUuid = jest.fn()
    .mockImplementationOnce(() => badUuid)
    .mockImplementationOnce(() => badUuid)
    .mockImplementationOnce(() => goodUuid);

  return {
    __esModule: true,
    default: mockUuid,
  };
});

describe('`modelUtils`', () => {
  describe('`isValidUuid`', () => {
    it('returns false if the uuid starts with `PDID`', () => {
      const result = isValidUuid('PDID1234');
      
      expect(result).toBe(false);
    });
    
    it('returns true if the uuid starts with anything except `PDID`', () => {
      const result = isValidUuid('pdid1234');
      
      expect(result).toBe(true);
    });
  });

  describe('`generateValidatedUuid`', () => {
    beforeEach(() => {
      jest.clearAllMocks();
    });

    it('does not create a new UUID starting with `PDID`', () => {
      expect(uuid).not.toHaveBeenCalled();
      const testUuid = generateValidatedUuid();
      expect(uuid).toHaveBeenCalledTimes(3);
      expect(testUuid).toBe(goodUuid);
    });
  });
});