import log from '@dazn/lambda-powertools-logger';
import { OK } from '@svc/lib/handlerUtils';
import { addSeenGclid, wasGclidSeen } from '@svc/lib/ddbRepos/idempotency-repo';
import { APIGatewayProxyEvent, Context } from 'aws-lambda';
import CallRail from '@svc/lib/models/CallRail';


export const handler = async (event: APIGatewayProxyEvent, ctx: Context) => {
  log.debug('Received event', { event });
  const callRailReq = JSON.parse(event.body!);
  log.debug('received callRail request', { callRailReq: callRailReq });

  const callRail = CallRail.fromCallRailRequest(callRailReq);
  const gclid = callRail.gclid

  const wasSeen = await wasGclidSeen(gclid, ctx.functionName);

  if (!wasSeen) {
    await callRail.put();

    await addSeenGclid(gclid, ctx.functionName);
  }
  
  return OK;
};