import log from '@dazn/lambda-powertools-logger';
import { OK } from '@svc/lib/handlerUtils';
import { addSeenCallRailSignature, wasCallRailSignatureSeen } from '@svc/lib/ddbRepos/idempotency-repo';
import { APIGatewayProxyEvent, Context } from 'aws-lambda';
import getCallRailSignature from '@svc/lib/getCallRailSignature';
import { CALLRAIL_SIGNING_KEY, lambdaConfig } from '@svc/config';


export const handler = async (event: APIGatewayProxyEvent, ctx: Context) => {
  log.debug('Received event', { event });
  
  // todo: add check to make sure reqType is valid/not missing
  const signature = event.headers.Signature;
  const signingKey = CALLRAIL_SIGNING_KEY;
  
  const body = event.body!;
  
  const isAuthorized = getCallRailSignature(body, signingKey) === signature;
  
  if (!isAuthorized) {
    return {
      statusCode: 401,
      body: JSON.stringify({})
    }
  }
  
  if (!event.queryStringParameters || !event.queryStringParameters.type) {
    return {
      statusCode: 400,
      body: JSON.stringify({message: "missing 'type' in query string parameters"}),
    }
  }
  
  
  const wasSeen = await wasCallRailSignatureSeen(signature, ctx.functionName);
  
  if (wasSeen) return OK;

  const reqType = event.queryStringParameters.type;

  const callRailReq = JSON.parse(body);
  log.debug('received activity request', { callRailReq });
  log.debug('type', {reqType})

  switch (reqType) {
    case 'postCall':
      const resp = await lambdaConfig.invokeSync('APIGWAddNewCallRail', body)
      await addSeenCallRailSignature(signature, ctx.functionName);
      return {
        statusCode: resp.StatusCode!,
        body: JSON.stringify({})
      }
  
    default:
      await addSeenCallRailSignature(signature, ctx.functionName);
      return {
        statusCode: 400,
        body: JSON.stringify({message: "invalid 'type'"})
      }
  }
};