import { Context, DynamoDBStreamEvent } from 'aws-lambda';
import log from '@dazn/lambda-powertools-logger';
import { Marshaller } from '@aws/dynamodb-auto-marshaller';
import { LowLevelClientProfile } from '@improovy/types';
import { addSeenDDBRecord, wasDDBRecordSeen } from '@svc/lib/ddbRepos/idempotency-repo';
import { PersonsApi, BasicPerson } from 'pipedrive';
import { PipedrivePersonFieldKeys } from '@improovy/types';
import { OK } from '@svc/lib/handlerUtils';

const marshaller = new Marshaller();

const pipedriveAPI = new PersonsApi();

export const handler = async (event: DynamoDBStreamEvent, ctx: Context) => {
  log.debug('Received event', { event });
  
  for (let record of event.Records) {
    const recordWasSeen = await wasDDBRecordSeen(record, ctx.functionName);
    if (recordWasSeen) continue;

    const client = marshaller.unmarshallItem(record.dynamodb!.NewImage!) as unknown as LowLevelClientProfile;

    if (client.isTest) continue;

    await updatePerson(client) 
    
    await addSeenDDBRecord(record, ctx.functionName);
  }
  return OK;
};

const updatePerson = async (client: LowLevelClientProfile, tries = 0) => {
  const update = BasicPerson.constructFromObject({
    [PipedrivePersonFieldKeys.uuid]: client.uuid,
    [PipedrivePersonFieldKeys.pipedriveName]: client.pipedriveName,
  });

  try {
    await pipedriveAPI.updatePerson(client.pdid, update);
  } catch (error) {
    if (tries > 2) throw error;
    
    await updatePerson(client, tries + 1)
  }
}