import { Context, DynamoDBStreamEvent } from 'aws-lambda';
import log from '@dazn/lambda-powertools-logger';
import { Marshaller } from '@aws/dynamodb-auto-marshaller';
import { addSeenDDBRecord, wasDDBRecordSeen } from '@svc/lib/ddbRepos/idempotency-repo';
import { DealsApi, UpdateDealRequest } from 'pipedrive';
import { PipedriveDealFieldKeys } from '@improovy/types';
import { OK } from '@svc/lib/handlerUtils';
import { LowLevelProjectDetails } from '@improovy/types';

const marshaller = new Marshaller();

const pipedriveAPI = new DealsApi();

export const handler = async (event: DynamoDBStreamEvent, ctx: Context) => {
  log.debug('Received event', { event });
  
  for (let record of event.Records) {
    const recordWasSeen = await wasDDBRecordSeen(record, ctx.functionName);
    if (recordWasSeen) continue;

    const project = marshaller.unmarshallItem(record.dynamodb!.NewImage!) as unknown as LowLevelProjectDetails;

    if (project.isTest) continue;
    
    await updateProject(project)
    
    await addSeenDDBRecord(record, ctx.functionName);
  }
  return OK;
};

const updateProject = async (project: LowLevelProjectDetails, tries = 0) => {
  const update = UpdateDealRequest.constructFromObject({
    [PipedriveDealFieldKeys.uuid]: project.uuid,
  });
  
  try {
    await pipedriveAPI.updateDeal(project.pdid, update);
  } catch (error) {
    if (tries > 2) throw error;
    
    await updateProject(project, tries + 1)
  }
}