import { Context, DynamoDBStreamEvent } from 'aws-lambda';
import log from '@dazn/lambda-powertools-logger';
import { Marshaller } from '@aws/dynamodb-auto-marshaller';
import { addSeenDDBRecord, wasDDBRecordSeen } from '@svc/lib/ddbRepos/idempotency-repo';
import { ActivitiesApi } from 'pipedrive';
import { PipedriveActivityFieldKeys } from '@improovy/types';
import { OK } from '@svc/lib/handlerUtils';
import { LowLevelActivity } from '@improovy/types';

const marshaller = new Marshaller();

const pipedriveAPI = new ActivitiesApi();

export const handler = async (event: DynamoDBStreamEvent, ctx: Context) => {
  log.debug('Received event', { event });
  
  for (let record of event.Records) {
    const recordWasSeen = await wasDDBRecordSeen(record, ctx.functionName);
    if (recordWasSeen) continue;

    const activity = marshaller.unmarshallItem(record.dynamodb!.NewImage!) as unknown as LowLevelActivity;

    if (activity.isTest) continue;

    await updateActivity(activity) 
    
    await addSeenDDBRecord(record, ctx.functionName);
  }
  return OK;
};

const updateActivity = async (activity: LowLevelActivity, tries = 0) => {
  if (activity.lastUpdatedField !== 'body') return;

  const update = {
    [PipedriveActivityFieldKeys.body]: activity.body
  };

  try {
    await pipedriveAPI.updateActivity(activity.pdid, update);
  } catch (error) {
    if (tries > 2) throw error;
    
    await updateActivity(activity, tries + 1)
  }
}