import log from '@dazn/lambda-powertools-logger';
import { OK } from '@svc/lib/handlerUtils';
import Activity from '@svc/lib/models/Activity';
import { addSeenPipedriveRequest, wasPipedriveRequestSeen } from '@svc/lib/ddbRepos/idempotency-repo';
import { APIGatewayProxyEvent, Context } from 'aws-lambda';


export const handler = async (event: APIGatewayProxyEvent, ctx: Context) => {
  log.debug('Received event', { event });
  const activityReq = JSON.parse(event.body!);
  log.debug('received activity request', { activityReq: activityReq });

  const wasSeen = await wasPipedriveRequestSeen(activityReq, ctx.functionName);

  if (!wasSeen) {
    let activity = await Activity.fromPipedrive(activityReq.current);
    
    await activity!.put();

    await addSeenPipedriveRequest(activityReq, ctx.functionName);
  }
  
  return OK;
};