import log from '@dazn/lambda-powertools-logger';
import Activity from '@svc/lib/models/Activity';
import { addSeenPipedriveRequest, wasPipedriveRequestSeen } from '@svc/lib/ddbRepos/idempotency-repo';
import { APIGatewayProxyEvent, Context } from 'aws-lambda';
import { lambdaConfig } from '@svc/config';

const OK = {
  statusCode: 200,
  body: JSON.stringify({}),
};

export const handler = async (event: APIGatewayProxyEvent, ctx: Context) => {
  log.debug('Received event', { event });
  const activityReq = JSON.parse(event.body!);
  log.debug('received activity request', { activityReq });

  const requestWasSeen = await wasPipedriveRequestSeen(activityReq, ctx.functionName);
  const activity = activityReq.current;
  
  if (!requestWasSeen) {

    const newActivity = Activity.fromPipedrive(activity);
    
    const oldActivity = await Activity.fetch(newActivity.pdid);

    if (!oldActivity) {
      const resp = await lambdaConfig.invokeSync('APIGWPipedriveAddActivity', event.body!);
      
      return {
        statusCode: resp.StatusCode!,
        body: JSON.stringify({}),
      };
    }

    const diffs = Activity.differingFields(newActivity, oldActivity);

    await newActivity.updateFields(diffs);
    
    await addSeenPipedriveRequest(activityReq, ctx.functionName);
  }


  
  return OK;
};