import log from '@dazn/lambda-powertools-logger';
import Client from '@svc/lib/models/Client';
import { addSeenPipedriveRequest, wasPipedriveRequestSeen } from '@svc/lib/ddbRepos/idempotency-repo';
import { APIGatewayProxyEvent, Context } from 'aws-lambda';

export const handler = async (event: APIGatewayProxyEvent, ctx: Context) => {
  log.debug('Received event', { event });
  const personReq = JSON.parse(event.body!);
  log.debug('received person request', { personReq });

  const requestWasSeen = await wasPipedriveRequestSeen(personReq, ctx.functionName);
  
  if (!requestWasSeen) {
    const client = Client.fromPipedrive(personReq.current);

    await client.put();

    await addSeenPipedriveRequest(personReq, ctx.functionName);
  }

  
  return {
    statusCode: 200,
    body: JSON.stringify({}),
  };
};