import log from '@dazn/lambda-powertools-logger';
import Client from '@svc/lib/models/Client';
import { addSeenPipedriveRequest, wasPipedriveRequestSeen } from '@svc/lib/ddbRepos/idempotency-repo';
import { APIGatewayProxyEvent, Context } from 'aws-lambda';
import { lambdaConfig } from '@svc/config';
import { OK } from '@svc/lib/handlerUtils';

export const handler = async (event: APIGatewayProxyEvent, ctx: Context) => {
  log.debug('Received event', { event });
  const personReq = JSON.parse(event.body!);
  log.debug('received person request', { personReq });

  const requestWasSeen = await wasPipedriveRequestSeen(personReq, ctx.functionName);
  
  if (!requestWasSeen) {
    const newClient = Client.fromPipedrive(personReq.current);
    const oldClient = await Client.fetchByPdid(newClient.pdid!);
    if (!oldClient) {
      const resp = await lambdaConfig.invokeSync('APIGWPipedriveAddPerson', event.body!);
      
      return {
        statusCode: resp.StatusCode!,
        body: JSON.stringify({}),
      };
    }

    log.debug('outside if, before diffs');

    const diffs = Client.differingFields(newClient, oldClient);

    log.debug('after diff, before updateFields');
    await newClient.updateFields(diffs);

    log.debug('after updatefields, before addseenpdreq');
    await addSeenPipedriveRequest(personReq, ctx.functionName);
  }

  
  return OK;
};