import { PIPEDRIVE_PASSWORD } from '@svc/config';
import log from '@dazn/lambda-powertools-logger';

const getResp = (isAuthorized: boolean, event: any) => {
  const Effect = isAuthorized ? 'Allow' : 'Deny';
  return {
    principalId: 'pipedrive',
    policyDocument: {
      Version: '2012-10-17',
      Statement: [
        {
          Action: 'execute-api:Invoke',
          Effect,
          Resource: event.methodArn.replace('/POST/', '/*/').replace(/\/pipedrive\/.+$/, '/pipedrive/*'),
        },
      ],
    },
  };
};


export const handler = async (event: any, ctx1: any, ctx2: any,) => {
  // TODO: remove this logger
  log.debug('event', {event});
  log.debug('ctx1', {ctx1});
  log.debug('ctx2', {ctx2});

  try {
    const isAuthorized = checkPipedriveAuth(event.authorizationToken.slice(6));
    return getResp(isAuthorized, event);
  } catch (e) {
    return e;
  }
};

const checkPipedriveAuth = (rawCredentials: string) => {
  const credentials = Buffer.from(rawCredentials, 'base64').toString().split(':');
  
  return (credentials[0] === 'pipedrive' && credentials[1] === PIPEDRIVE_PASSWORD);
};