import log from '@dazn/lambda-powertools-logger';
import Project from '@svc/lib/models/Project';
import { addSeenPipedriveRequest, wasPipedriveRequestSeen } from '@svc/lib/ddbRepos/idempotency-repo';
import { OK } from '@svc/lib/handlerUtils';
import { APIGatewayProxyEvent, Context } from 'aws-lambda';

export const handler = async (event: APIGatewayProxyEvent, ctx: Context) => {
  log.debug('Received event', { event });
  const dealReq = JSON.parse(event.body!);
  log.debug('received deal request', { dealReq });

  const requestWasSeen = await wasPipedriveRequestSeen(dealReq, ctx.functionName);
  const deal = dealReq.previous;
  
  if (!requestWasSeen && Project.dealIsProject(deal)) {

    const newProject = await Project.fetchByPdid(dealReq.meta.id);

    if (newProject) {
      await newProject.softDelete()
    }
    
    await addSeenPipedriveRequest(dealReq, ctx.functionName);
  }
  
  return OK;
};