import { QueryCommand, QueryCommandInput, UpdateCommand, UpdateCommandInput, GetCommandInput, GetCommand } from '@aws-sdk/lib-dynamodb';
import { ddb } from '@svc/config';
// import { LowLevelActivity } from '@improovy/types';
// import { LowLevelClientProfile } from '../improovy';
// import { LowLevelProjectDetails } from '../improovy';
// import log from '@dazn/lambda-powertools-logger';
// import { LowLevelCallRail } from '@improovy/types';

// todo: add ability to delete more than 25 items at once/handle paginating
export const queryByPK = async (TableName: string, pk: string, sk?: string): Promise<{ [key: string]: any }[]> => {
  const ExpressionAttributeValues: {[key: string]: any} = {':pk': pk}
  let KeyConditionExpression = 'pk = :pk'

  if (sk) {
    ExpressionAttributeValues[':sk'] = sk;
    KeyConditionExpression += ' and begins_with(sk, :sk)'
  }
  const params: QueryCommandInput = {
    TableName,
    ExpressionAttributeValues,
    KeyConditionExpression
  };

  const ddbResp = await ddb.documentClient.send(new QueryCommand(params));
  return ddbResp.Items!;
};

// export const getItem = async (TableName: string, pk: string, sk?: string): Promise<{ [key: string]: any } | null> => {
//   const Key: {pk: string, sk?: string} = { pk }

//   if (sk) Key.sk = sk;

//   const params: GetCommandInput = {
//     TableName,
//     Key,
//   };

//   const ddbResp = await ddb.documentClient.send(new GetCommand(params));
//   return ddbResp.Item || null;
// };

export const updateItem = async <Type extends {pk: string, sk: string, lastUpdatedField?: string, changeSource: string, [key: string]: any}>(item: Type, TableName: string, secondaryIndices?: {indexNum: number, pk?: string, sk?: string}) => {
  const { pk, sk, lastUpdatedField, changeSource } = item;

  if (!lastUpdatedField) throw new Error("missing lastUpdatedField");
  

  const val = item[lastUpdatedField as keyof Type];

  let UpdateExpression = 'set lastUpdatedField = :lastUpdatedField, changeSource = :changeSource';
  let ExpressionAttributeValues: {[key: string]: any} = {
    ':lastUpdatedField': lastUpdatedField,
    ':changeSource': changeSource
  }

  let indexNum;

  if (secondaryIndices) {
    ({ indexNum } = secondaryIndices);
  }

  const isRemoving = val === null || val === undefined;

  if (isRemoving) {
    UpdateExpression += ` remove ${lastUpdatedField}`

    if (secondaryIndices) {
      UpdateExpression += `, gsi${indexNum}_pk, gsi${indexNum}_sk`
    }
  } else {
    UpdateExpression += `, ${lastUpdatedField} = :val`
    ExpressionAttributeValues[':val'] = val;

    if (secondaryIndices) {
      const { pk, sk } = secondaryIndices;
      const pkKey = `:pk${indexNum}`
      const skKey = `:sk${indexNum}`

      UpdateExpression += `, gsi${indexNum}_pk = ${pkKey}, gsi${indexNum}_sk = ${skKey}`;

      ExpressionAttributeValues[pkKey] = pk;
      ExpressionAttributeValues[skKey] = sk;
    }
  }

  const params: UpdateCommandInput = {
    TableName,
    Key: {
      pk,
      sk,
    },
    UpdateExpression,
    ExpressionAttributeValues,
  };

  return ddb.documentClient.send(new UpdateCommand(params));
};

export const removeProperty = async <Type extends {pk: string, sk?: string, lastUpdatedField: string}>(item: Type, TableName: string) => {
  const { pk, sk, lastUpdatedField } = item;
  
  const params = {
    TableName,
    Key: {
      pk,
      sk,
    },
    UpdateExpression: `remove ${lastUpdatedField} set lastUpdatedField = :lastUpdatedField`,
    ExpressionAttributeValues: {
      ':lastUpdatedField': lastUpdatedField,
    },
  };

  return ddb.documentClient.send(new UpdateCommand(params));
};

export const queryByGSI = async (TableName: string, indexNumber: number, pk: string, sk?: string) => {
  const params: QueryCommandInput = {
    TableName,
    IndexName: `gsi_${indexNumber}`,
    ExpressionAttributeValues: {
      ':pk': pk,
    },
    KeyConditionExpression: `gsi${indexNumber}_pk = :pk`,
  };

  // if (sk) {
  //   params.ExpressionAttributeValues![':sk'] = sk;
  //   params.KeyConditionExpression = `gsi${indexNumber}_pk = :pk and begins_with(gsi${indexNumber}_sk, :sk)`
  // }

  const ddbResp = await ddb.documentClient.send(new QueryCommand(params));
  return ddbResp.Items!;
};