import formatPhoneNumber from '@svc/lib/formatPhoneNumber';
import { ChangeSource } from '@improovy/types';
import { CallRailFieldKeys, CallRailInterface, CallRailRequestBody, MutableCallRailProperties } from '@improovy/types';
import { fetchCallRailInterface, hardDeleteCallRail, putCallRail, updateCallRail } from '../ddbRepos/callRail-repo';

export default class CallRail implements CallRailInterface {
  #gclid: string;

  #createdAt: number;

  #isAttributed: boolean;

  #phoneNumber: string;

  changeSource: ChangeSource;

  #isTest?: boolean;

  #lastUpdatedField?: MutableCallRailProperties

  static fromCallRailRequest(req: CallRailRequestBody) {
    const crInterface: CallRailInterface = {
      gclid: req[CallRailFieldKeys.gclid],
      createdAt: Date.parse(req[CallRailFieldKeys.timestamp]),
      phoneNumber: formatPhoneNumber(req[CallRailFieldKeys.phoneNumber]),
      changeSource: ChangeSource.callRail,
    }

    const isTest = req.isTest;

    if (isTest) crInterface.isTest = isTest;

    return new CallRail(crInterface)
  }

  static async fetch(phoneNumber: string) {
    const callRailInterface = await fetchCallRailInterface(phoneNumber);
    if (!callRailInterface) return null;
    return new CallRail(callRailInterface);
  }

  constructor(callRailInterface: CallRailInterface) {
    this.#gclid = callRailInterface.gclid;
    this.#isAttributed = callRailInterface.isAttributed || false;
    this.#createdAt = callRailInterface.createdAt;
    this.#phoneNumber = formatPhoneNumber(callRailInterface.phoneNumber);
    this.changeSource = callRailInterface.changeSource;
    if (callRailInterface.isTest) this.#isTest = callRailInterface.isTest;
    if (callRailInterface.lastUpdatedField) this.#lastUpdatedField = callRailInterface.lastUpdatedField;
  }

  async put() {
    return putCallRail(this);
  }

  async hardDelete() {
    return hardDeleteCallRail(this);
  }

  updateIsAttributed(isAttributed: boolean) {
    this.#lastUpdatedField = MutableCallRailProperties.isAttributed;
    this.#isAttributed = isAttributed;
    return updateCallRail(this);
  }

  get isTest() {
    return this.#isTest || false
  }

  get createdAt() {
    return this.#createdAt;
  }

  get phoneNumber() {
    return this.#phoneNumber;
  }

  get gclid() {
    return this.#gclid;
  }

  get isAttributed() {
    return this.#isAttributed;
  }

  get lastUpdatedField() {
    return this.#lastUpdatedField;
  }
  
}