import { ActivityType } from './activities';
import { ClientIncomingContactMethod } from './clients';

export enum PipedriveEventEntities {
  activity = 'activity',
  deal = 'deal',
  person = 'person',
}

export enum PipedriveEventActions {
  updated = 'updated',
  added = 'added',
  merged = 'merged',
  deleted = 'deleted',
}

export interface PipedrivePerson {
  id: number | string,
  [PipedrivePersonFieldKeys.gclid]?: string,
  [PipedrivePersonFieldKeys.uuid]?: string,
  [PipedrivePersonFieldKeys.createdAt]: string,
  [PipedrivePersonFieldKeys.incomingContactMethod]: keyof InboundIncomingContactMethodMap | null,
  [PipedrivePersonFieldKeys.phones]: { value: string }[],
  [PipedrivePersonFieldKeys.emails]: { value: string }[],
  [PipedrivePersonFieldKeys.pipedriveName]: string,
  isTest?: boolean,
}

export enum PipedrivePersonFieldKeys {
  gclid = '8aa12c2006b2a2a6964b69679969b800f8df63b0',
  uuid = '3fd4a63fda1ead2fdb8d0ef6c8eef7cfd0ac8ab1',
  incomingContactMethod = '88a4f838702be0a47df31fb237ccd732898930a3',
  pdid = 'id',
  createdAt = 'add_time',
  emails = 'email',
  phones = 'phone',
  pipedriveName = 'name',
}

export interface PipedriveDeal {
  id: number | string,
  [PipedriveDealFieldKeys.gclid]?: string,
  [PipedriveDealFieldKeys.uuid]?: string,
  [PipedriveDealFieldKeys.createdAt]: string,
  [PipedriveDealFieldKeys.gaTransactionID]?: string,
  [PipedriveDealFieldKeys.personPdid]: number | string,
  [PipedriveDealFieldKeys.pipeline]: number | string,
  [PipedriveDealFieldKeys.stage]: number | string,
  [PipedriveDealFieldKeys.clientCost]?: number | string,
  [PipedriveDealFieldKeys.projectStatus]: PipedriveDealStatus,
  isTest?: boolean
}

export interface PipedriveActivity {
  [PipedriveActivityFieldKeys.pdid]: number | string,
  [PipedriveActivityFieldKeys.body]: string,
  [PipedriveActivityFieldKeys.projectPdid]: number | string,
  [PipedriveActivityFieldKeys.activityType]: ActivityType,
  isTest?: boolean,
}

export enum PipedriveActivityFieldKeys {
  pdid = 'id',
  body = 'note',
  projectPdid = 'deal_id',
  activityType = 'type',
}

export enum PipedriveDealStatus {
  open = 'open',
  lost = 'lost',
  won = 'won',
  deleted = 'deleted',
}

export enum PipedriveDealFieldKeys {
  gclid = 'd02d5eade3c980104c7ceb3452c3f19457655786',
  uuid = 'e4b1a5fdc67bdbc43f89d319658446f68f24edf6',
  gaTransactionID = '23765b92370cb73c419a64f7c75ca77ed94f1172',
  personPdid = 'person_id',
  pipeline = 'pipeline_id',
  stage = 'stage_id',
  clientCost = 'value',
  createdAt = 'add_time',
  projectStatus = 'status',
  pdid = 'id',
  clientPdid = 'person_id',
}

export interface InboundIncomingContactMethodMap {
  '238': ClientIncomingContactMethod.PHONE,
  '239': ClientIncomingContactMethod.WEBFORM,
  '241': ClientIncomingContactMethod.EMAIL,
  '242': ClientIncomingContactMethod.INTEGRATION,
  '243': ClientIncomingContactMethod.UNKNOWN,
}

export const inboundIncomingContactMethodMap: InboundIncomingContactMethodMap = {
  '238': ClientIncomingContactMethod.PHONE,
  '239': ClientIncomingContactMethod.WEBFORM,
  '241': ClientIncomingContactMethod.EMAIL,
  '242': ClientIncomingContactMethod.INTEGRATION,
  '243': ClientIncomingContactMethod.UNKNOWN,
} as const;

export interface OutboundIncomingContactMethodMap {
  [ClientIncomingContactMethod.PHONE]: '238',
  [ClientIncomingContactMethod.WEBFORM]: '239',
  [ClientIncomingContactMethod.EMAIL]: '241',
  [ClientIncomingContactMethod.INTEGRATION]: '242',
  [ClientIncomingContactMethod.UNKNOWN]: '243',
};

export const outboundIncomingContactMethodMap: OutboundIncomingContactMethodMap = {
  [ClientIncomingContactMethod.PHONE]: '238',
  [ClientIncomingContactMethod.WEBFORM]: '239',
  [ClientIncomingContactMethod.EMAIL]: '241',
  [ClientIncomingContactMethod.INTEGRATION]: '242',
  [ClientIncomingContactMethod.UNKNOWN]: '243',
} as const;

export interface PipedriveDealRequest extends PipedriveRequest {
  current: PipedriveDeal,
  meta: PipedriveMeta,
}

interface PipedriveMeta {
  id: string | number,
  object: PipedriveEventEntities,
  action: PipedriveEventActions,
  timestamp_micro: number | string,
  isTest?: boolean,
}

export interface PipedrivePersonRequest extends PipedriveRequest {
  current: PipedrivePerson,
}

export interface PipedriveActivityRequest extends PipedriveRequest {
  current: PipedriveActivity
}

export interface GetTestPipedriveActivityInput {
  fieldsToGenerate?: string[],
  overrides?: Partial<PipedriveActivity>,
  bodyOptions?: {
    type: "callSid",
    override?: string,
  }
}
export interface PipedriveRequest {
  [key: string]: any,
  meta: PipedriveMeta
}