import { apiGatewayConfig, AWS_REGION, ddb } from '@svc/config';
import { handler } from '@svc/handlers/callRail/addNewCallRail';
import { buildLowLevelCallRail } from '@svc/lib/ddbRepos/callRail-repo';
import { ApiGatewayHandlerInvoker } from '@tests/utils/handler-invokers/ApiGatewayHandlerInvoker';
import TestDataGenerator from '@tests/utils/testDataGenerator';
import _ from 'lodash';
import CallRail from '@svc/lib/models/CallRail';

const testDataGenerator = new TestDataGenerator();

const apiInvoker = new ApiGatewayHandlerInvoker({
  baseUrl: apiGatewayConfig.getBaseUrl(),
  handler,
});

const awsTestOptions = {
  region: AWS_REGION,
  table: ddb.callRailTable,
};

describe('`addCallRail`', () => {
  it('adds the callRail from CallRail to the callRail table', async () => {
    const req = testDataGenerator.getCallRailRequest();

    const testCallRail = CallRail.fromCallRailRequest(req.body);

    const details = buildLowLevelCallRail(testCallRail);

    const response = await apiInvoker.invoke({
      event: {
        pathTemplate: 'callrail?type=postCall',
        httpMethod: 'POST',
        ...req
      },
    });
    
    expect(response.statusCode).toEqual(200);

    await expect({...awsTestOptions, timeout: 50000}).toHaveItem(_.pick(details, ['pk', 'sk']));
  });
  
  afterAll(() => {
    return testDataGenerator.deleteAll();
  });
});