import { AWS_REGION, lambdaConfig } from '@svc/config';
import { buildLowLevelClient } from '@svc/lib/ddbRepos/clients-repo-utils';
import TestDataGenerator from '@tests/utils/testDataGenerator';
import { waitForLambdaLogs } from '@tests/utils/wait';

const testDataGenerator = new TestDataGenerator();

const lambdaFunctionName = `${lambdaConfig.functionNamePrefix}DDBSyncNewClient`;

const awsTestOptions = {
  region: AWS_REGION,
  function: lambdaFunctionName,
};

describe('`syncNewClient` Lambda function', () => {
  it('is triggered when a new client is inserted into the Clients DDB table', async () => {
    const testClient = testDataGenerator.getClient();
    await testClient.put();
    const expectedLog = testClient.uuid;
    return expect({
      ...awsTestOptions,
      timeout: 50000,
    }).toHaveLog(expectedLog);

  });

  it('is not triggered when an entity of type other than client is added to the database', async () => {
    const testProject = testDataGenerator.getProject({ fieldsToGenerate: ['all'] });
    await testProject.put();

    const unexpectedProjectLog = testProject.gclid!;

    const testClient = testDataGenerator.getClient();
    await testClient.put();

    const lowLevelClient = buildLowLevelClient(testClient);
    const unexpectedContactLog = lowLevelClient.contactMethods[0].sk;


    await waitForLambdaLogs();
    
    await expect(awsTestOptions).not.toHaveLog(unexpectedProjectLog);
    await expect(awsTestOptions).not.toHaveLog(unexpectedContactLog);
  });

  afterAll(async () => testDataGenerator.deleteAll());
});