import { AWS_REGION, lambdaConfig } from '@svc/config';
import { buildLowLevelClient } from '@svc/lib/ddbRepos/clients-repo-utils';
import TestDataGenerator from '@tests/utils/testDataGenerator';
import { waitForLambdaLogs } from '@tests/utils/wait';

const testDataGenerator = new TestDataGenerator();

const lambdaFunctionName = `${lambdaConfig.functionNamePrefix}DDBSyncNewProject`;

const awsTestOptions = {
  region: AWS_REGION,
  function: lambdaFunctionName,
};

describe('`syncNewProject` Lambda function', () => {
  it('is triggered when a new project is inserted into the Projects DDB table', async () => {
    const testProject = testDataGenerator.getProject();
    await testProject.put();
    const expectedLog = testProject.uuid;
    return expect({
      ...awsTestOptions,
      timeout: 50000,
    }).toHaveLog(expectedLog);

  });

  it('is not triggered when an entity of type other than client is added to the database', async () => {
    // todo: test for other entities once added to this table
    const testClient = testDataGenerator.getClient();
    await testClient.put();

    const lowLevelClient = buildLowLevelClient(testClient);

    await waitForLambdaLogs();
    
    await expect(awsTestOptions).not.toHaveLog(lowLevelClient.profile.pk);
  });

  afterAll(async () => testDataGenerator.deleteAll());
});