import { AWS_REGION, lambdaConfig, TEST_ACTIVITY_PDID } from '@svc/config';
import { ChangeSource } from '@improovy/types';
import TestDataGenerator from '@tests/utils/testDataGenerator';
import { waitForPipedriveUpdate } from '@tests/utils/wait';
import { ActivitiesApi } from 'pipedrive';

const testDataGenerator = new TestDataGenerator();

const lambdaFunctionName = `${lambdaConfig.functionNamePrefix}DDBSyncUpdatedActivity`;

const awsTestOptions = {
  region: AWS_REGION,
  function: lambdaFunctionName,
};

const pipedriveApi = new ActivitiesApi();

describe('`syncUpdatedActivity` Lambda function', () => {
  it('is triggered when an activity is updated whose changeSource is aws', async () => {
    const testActivity = testDataGenerator.getActivity();
    await testActivity.put();

    testActivity.changeSource = ChangeSource.aws
    testActivity.updateBody('test body');
    const expectedLog = testActivity.pdid;
    return expect({
      ...awsTestOptions,
      timeout: 50000,
    }).toHaveLog(expectedLog);

  });

  it('updates the pipedrive activity', async () => {
    const testActivity = testDataGenerator.getActivity({overrides: {pdid: `${TEST_ACTIVITY_PDID}`, isTest: false}});
    await testActivity.put();

    testActivity.changeSource = ChangeSource.aws
    testActivity.updateBody('test body');

    await waitForPipedriveUpdate();

    const pdActivity = await pipedriveApi.getActivity(TEST_ACTIVITY_PDID)

    return expect(pdActivity.data.note).toBe('test body');
  });

  afterAll(async () => {
    await testDataGenerator.deleteAll();
    await pipedriveApi.updateActivity(TEST_ACTIVITY_PDID, {note: ''})
  });
});