import { apiGatewayConfig, AWS_REGION, ddb, pipedriveAuthHeader } from '@svc/config';
import { handler } from '@svc/handlers/pipedrive/activities/addPipedriveActivityToActivities';
import { buildLowLevelActivity } from '@svc/lib/ddbRepos/activities-repo';
import { ApiGatewayHandlerInvoker } from '@tests/utils/handler-invokers/ApiGatewayHandlerInvoker';
import TestDataGenerator from '@tests/utils/testDataGenerator';
import _ from 'lodash';
import Activity from '@svc/lib/models/Activity';

const testDataGenerator = new TestDataGenerator();

const apiInvoker = new ApiGatewayHandlerInvoker({
  baseUrl: apiGatewayConfig.getBaseUrl(),
  handler,
});

const awsTestOptions = {
  region: AWS_REGION,
  table: ddb.activitiesTable,
};

describe('`addPipedriveActivityToActivities`', () => {
  it('adds the activity from pipedrive to the activities table', async () => {
    const req = testDataGenerator.getPipedriveActivityRequestBody();
    const testActivityBody = req.current;

    const testActivity = Activity.fromPipedrive(testActivityBody);

    const details = buildLowLevelActivity(testActivity);

    const response = await apiInvoker.invoke({
      event: {
        pathTemplate: 'pipedrive/activities/new',
        httpMethod: 'POST',
        body: JSON.stringify(req),
        headers: pipedriveAuthHeader,
      },
    });
    
    expect(response.statusCode).toEqual(200);

    await expect(awsTestOptions).toHaveItem(_.pick(details, ['pk', 'sk']));
  });
  
  afterAll(() => {
    return testDataGenerator.deleteAll();
  });
});