import { apiGatewayConfig, AWS_REGION, ddb, pipedriveAuthHeader  } from '@svc/config';
import { handler } from '@svc/handlers/pipedrive/activities/updateActivityFromPipedriveActivity';
import Activity from '@svc/lib/models/Activity';
import { buildLowLevelActivity } from '@svc/lib/ddbRepos/activities-repo';
import { PipedriveActivityFieldKeys } from '@improovy/types';
import { ApiGatewayHandlerInvoker } from '@tests/utils/handler-invokers/ApiGatewayHandlerInvoker';
import TestDataGenerator, { generateCallSidBody } from '@tests/utils/testDataGenerator';
import { waitForGSIUpdates } from '@tests/utils/wait';
import _ from 'lodash';

const testDataGenerator = new TestDataGenerator();

const apiInvoker = new ApiGatewayHandlerInvoker({
  baseUrl: apiGatewayConfig.getBaseUrl(),
  handler,
});

const awsTestOptions = {
  region: AWS_REGION,
  table: ddb.activitiesTable,
};

describe('`updateActivityFromPipedriveActivity`', () => {
  it('updates the activity from pipedrive in the activities table', async () => {
    const testActivityReqOld = testDataGenerator.getPipedriveActivityRequestBody();
    const testActivityOld = Activity.fromPipedrive(testActivityReqOld.current);
    await testActivityOld.put();
    await waitForGSIUpdates();
    const newCallSid = 'testCallSid';
    const newBody = generateCallSidBody(newCallSid)
    
    const testActivityReqNew = testDataGenerator.getPipedriveActivityRequestBody({ overrides: { [PipedriveActivityFieldKeys.pdid]: testActivityOld.pdid, [PipedriveActivityFieldKeys.body]: newBody, [PipedriveActivityFieldKeys.projectPdid]: testActivityOld.projectPdid, [PipedriveActivityFieldKeys.activityType]: testActivityOld.activityType }});
    
    const newTestActivity = Activity.fromPipedrive(testActivityReqNew.current);
    const lowLevelTestActivity = buildLowLevelActivity(newTestActivity);

    const response = await apiInvoker.invoke({
      event: {
        pathTemplate: 'pipedrive/activities/update',
        httpMethod: 'POST',
        body: JSON.stringify(testActivityReqNew),
        headers: pipedriveAuthHeader,
      },
    });

    expect(response.statusCode).toEqual(200);

    await waitForGSIUpdates();

    await expect(awsTestOptions).toHaveItem(_.pick(lowLevelTestActivity, ['pk', 'sk']), expect.objectContaining({ ...lowLevelTestActivity, callSid: newCallSid, body: newBody}));
  });

  it('adds the activity if it is not yet in the table', async () => {
    const testActivityReq = testDataGenerator.getPipedriveActivityRequestBody();
    const testActivity = Activity.fromPipedrive(testActivityReq.current);
    
    const lowLevelTestActivity = buildLowLevelActivity(testActivity);

    const response = await apiInvoker.invoke({
      event: {
        pathTemplate: 'pipedrive/activities/update',
        httpMethod: 'POST',
        body: JSON.stringify(testActivityReq),
        headers: pipedriveAuthHeader,
      },
    });

    expect(response.statusCode).toEqual(200);

    await waitForGSIUpdates();

    await expect(awsTestOptions).toHaveItem(_.pick(lowLevelTestActivity, ['pk', 'sk']), lowLevelTestActivity);
  });

  afterAll(() => {
    return testDataGenerator.deleteAll();
  });
});