import { apiGatewayConfig, AWS_REGION, ddb, pipedriveAuthHeader  } from '@svc/config';
import { handler } from '@svc/handlers/pipedrive/clients/addPipedrivePersonToClients';
import Client from '@svc/lib/models/Client';
import { buildLowLevelClient } from '@svc/lib/ddbRepos/clients-repo-utils';
import { ApiGatewayHandlerInvoker } from '@tests/utils/handler-invokers/ApiGatewayHandlerInvoker';
import TestDataGenerator from '@tests/utils/testDataGenerator';
import _ from 'lodash';

const testDataGenerator = new TestDataGenerator();

const apiInvoker = new ApiGatewayHandlerInvoker({
  baseUrl: apiGatewayConfig.getBaseUrl(),
  handler,
});

const awsTestOptions = {
  region: AWS_REGION,
  table: ddb.clientsTable,
};

describe('`addPipedrivePersonToClients`', () => {
  it('adds the person from pipedrive to the clients table', async () => {
    const req = testDataGenerator.getPipedrivePersonRequestBody();
    const testPerson = req.current;
    
    // needed to so the created client is deleted
    const testClient = Client.fromPipedrive(testPerson);
    const { profile } = buildLowLevelClient(testClient);
    const response = await apiInvoker.invoke({
      event: {
        pathTemplate: 'pipedrive/clients/new',
        httpMethod: 'POST',
        body: JSON.stringify(req),
        headers: pipedriveAuthHeader,
      },
    });

    expect(response.statusCode).toEqual(200);

    await expect(awsTestOptions).toHaveItem(_.pick(profile, ['pk', 'sk']), profile);
  });

  afterAll(() => {
    return testDataGenerator.deleteAll();
  });
});