import { apiGatewayConfig, AWS_REGION, ddb, pipedriveAuthHeader  } from '@svc/config';
import { handler } from '@svc/handlers/pipedrive/clients/updateClientFromPipedrivePerson';
import Client from '@svc/lib/models/Client';
import { buildLowLevelClient } from '@svc/lib/ddbRepos/clients-repo-utils';
import { PipedrivePersonFieldKeys } from '@improovy/types';
import { ApiGatewayHandlerInvoker } from '@tests/utils/handler-invokers/ApiGatewayHandlerInvoker';
import TestDataGenerator from '@tests/utils/testDataGenerator';
import { waitForGSIUpdates } from '@tests/utils/wait';
import _ from 'lodash';

const testDataGenerator = new TestDataGenerator();

const apiInvoker = new ApiGatewayHandlerInvoker({
  baseUrl: apiGatewayConfig.getBaseUrl(),
  handler,
});

const awsTestOptions = {
  region: AWS_REGION,
  table: ddb.clientsTable,
};

describe('`updateClientFromPipedrivePerson`', () => {
  it('updates the person from pipedrive in the clients table', async () => {
    const testPersonReqOld = testDataGenerator.getPipedrivePersonRequestBody();
    const testPersonOld = Client.fromPipedrive(testPersonReqOld.current);
    await testPersonOld.put();
    await waitForGSIUpdates();
    const newGclid = 'test_gclid';
    
    const testPersonReqNew = testDataGenerator.getPipedrivePersonRequestBody({ [PipedrivePersonFieldKeys.uuid]: testPersonOld.uuid, [PipedrivePersonFieldKeys.gclid]: newGclid, [PipedrivePersonFieldKeys.pdid]: testPersonOld.pdid, [PipedrivePersonFieldKeys.pipedriveName]: testPersonOld.pipedriveName });
    
    const newTestPerson = Client.fromPipedrive(testPersonReqNew.current);
    const lowLevelTestClient = buildLowLevelClient(newTestPerson);

    const response = await apiInvoker.invoke({
      event: {
        pathTemplate: 'pipedrive/clients/update',
        httpMethod: 'POST',
        body: JSON.stringify(testPersonReqNew),
        headers: pipedriveAuthHeader,
      },
    });

    expect(response.statusCode).toEqual(200);

    await waitForGSIUpdates();

    await expect(awsTestOptions).toHaveItem(_.pick(lowLevelTestClient.profile, ['pk', 'sk']), { ...lowLevelTestClient.profile, gclid: newGclid, lastUpdatedField: 'gclid' });
  });

  it('if the person is not yet in the clients table, it adds them', async () => {
    const testPersonReq = testDataGenerator.getPipedrivePersonRequestBody();
    const testPerson = Client.fromPipedrive(testPersonReq.current);
    
    const lowLevelTestClient = buildLowLevelClient(testPerson);

    const response = await apiInvoker.invoke({
      event: {
        pathTemplate: 'pipedrive/clients/update',
        httpMethod: 'POST',
        body: JSON.stringify(testPersonReq),
        headers: pipedriveAuthHeader,
      },
    });

    expect(response.statusCode).toEqual(200);

    await waitForGSIUpdates();

    await expect(awsTestOptions).toHaveItem(_.pick(lowLevelTestClient.profile, ['pk', 'sk']), lowLevelTestClient.profile);
  });

  afterAll(() => {
    return testDataGenerator.deleteAll();
  });
});