import CallRail from '@svc/lib/models/CallRail';
import TestDataGenerator from '@tests/utils/testDataGenerator';
import { CallRailFieldKeys, CallRailInterface } from '@improovy/types';
import { ChangeSource } from '@improovy/types';
import formatPhoneNumber from '@svc/lib/formatPhoneNumber';

const testDataGenerator = new TestDataGenerator(false);

jest.mock('@svc/lib/ddbRepos/callRail-repo', () => {
  return {
    putCallRail: jest.fn(() => {}),
    fetchCallRailInterface: jest.fn(),
    updateCallRail: jest.fn(),
    removeCallRailProperty: jest.fn(),
  };
});


describe('`CallRail` model', () => {
  it('correctly maps the fields when created with a CallRailInterface', () => {
    const testCallRailInterface = testDataGenerator.getCallRailInterface({ fieldsToGenerate: ['all'] });
    // if this assertion fails, it's likely because the callRailInterface was changed.
    // Update the expected number of properties and add tests to make sure they are
    // mapped correctly
    expect(Object.keys(testCallRailInterface)).toHaveLength(7);

    const testCallRail = new CallRail({...testCallRailInterface, phoneNumber: `+1${testCallRailInterface.phoneNumber}`});

    for (let key in testCallRailInterface) {
      expect(testCallRail[key as keyof CallRail]).toBe(testCallRailInterface[key as keyof CallRailInterface]);
    }
  });

  it('correctly assigns default values when values are missing', () => {
    const testCallRail = testDataGenerator.getCallRail();

    expect(testCallRail.isAttributed).toBe(false);
  });

  describe('`Activity::fromCallRailRequest`', () => {
    it('correctly maps in a CallRail record from a CallRail request', () => {
      const testCallRailRequest = testDataGenerator.getCallRailRequestBody()

      const gclid = testCallRailRequest[CallRailFieldKeys.gclid];
      const createdAt = Date.parse(testCallRailRequest[CallRailFieldKeys.timestamp]);
      const phoneNumber = formatPhoneNumber(testCallRailRequest[CallRailFieldKeys.phoneNumber]);
      const isTest = true;
      const changeSource = ChangeSource.callRail;

      const testCallRail = CallRail.fromCallRailRequest(testCallRailRequest);

      expect(testCallRail.gclid).toBe(gclid);
      expect(testCallRail.createdAt).toBe(createdAt);
      expect(testCallRail.phoneNumber).toBe(phoneNumber);
      expect(testCallRail.isTest).toBe(isTest);
      expect(testCallRail.changeSource).toBe(changeSource);
      expect(testCallRail.isAttributed).toBe(false);
    })
  })
});