import { QueryCommand } from '@aws-sdk/lib-dynamodb';
import { ddb } from '@svc/config';
import { queryByPK } from '@svc/lib/ddbRepos/repo-utils';
import TestDataGenerator from '@tests/utils/testDataGenerator';

const testDataGenerator = new TestDataGenerator(false);

jest.mock('@svc/config', () => {
  const original = jest.requireActual('@svc/config');

  return {
    __esModule: true,
    ddb: {
      ...original.ddb,
      documentClient: {
        send: jest.fn().mockImplementation(() => {
          return { Items: [] };
        }),
      },
    },
  };
});

describe('`repo-utils`', () => {
  describe('`queryByPK`', () => {
    it('invokes the documentClient send method with a correctly formatted QueryCommand command', async () => {
      const mockSend = ddb.documentClient.send;
      const testClient = testDataGenerator.getClient();
      
      
      expect(mockSend).not.toHaveBeenCalled();      
      queryByPK(ddb.clientsTable, testClient.uuid);
      expect(mockSend).toHaveBeenCalledTimes(1);      
      
      // @ts-ignore
      const mockSendInput = mockSend.mock.calls[0][0];

      const testParams = {
        TableName: ddb.clientsTable,
        ExpressionAttributeValues: {
          ':pk': testClient.uuid,
        },
        KeyConditionExpression: 'pk = :pk',
      };

      const expectedInput = new QueryCommand(testParams);

      expect(JSON.stringify(mockSendInput)).toBe(JSON.stringify(expectedInput));
    });
  });
});