import { Marshaller } from '@aws/dynamodb-auto-marshaller';
import { AWS_REGION, lambdaConfig, TEST_PERSON_PDID } from '@svc/config';
import { handler } from '@svc/handlers/outbound/syncNewClient';
import { buildLowLevelClient } from '@svc/lib/ddbRepos/clients-repo-utils';
import { PipedrivePersonFieldKeys } from '@improovy/types';
import uuid from '@svc/lib/uuid';
import getLambdaContext from '@tests/utils/getLambdaContext';
import TestDataGenerator from '@tests/utils/testDataGenerator';
import { DynamoDBStreamEvent } from 'aws-lambda';
import { PersonsApi, BasicPerson } from 'pipedrive';


jest.mock('pipedrive', () => {
  const original = jest.requireActual('pipedrive');
  const updatePerson = jest.fn();
  return {
    __esModule: true,
    ...original,
    PersonsApi: jest.fn().mockImplementation(() => {
      return { updatePerson };
    }),
  };
});

jest.mock('@svc/lib/ddbRepos/idempotency-repo', () => {
  return {
    __esModule: true,
    wasDDBRecordSeen: () => false,
    addSeenDDBRecord: () => Promise.resolve(),
  };
});

const marshaller = new Marshaller();

const testDataGenerator = new TestDataGenerator(false);

const lambdaFunctionName = `${lambdaConfig.functionNamePrefix}DDBSyncNewClient`;

const awsTestOptions = {
  region: AWS_REGION,
  function: lambdaFunctionName,
};

describe('`syncNewClient` Lambda function', () => {
   
  it('updates the pipedrive person with the uuid', async () => {
    const testClient = testDataGenerator.getClient({ overrides: { pdid: TEST_PERSON_PDID.toString(), pipedriveName: 'LongtermTesting DoNotDelete' }, fieldsToExclude: ['isTest'] });
    const lowLevelClient = buildLowLevelClient(testClient);
    const profile = lowLevelClient.profile;
    const marshalledProfile = marshaller.marshallItem(profile);

    const testEvent: DynamoDBStreamEvent = {
      Records: [
        {
          eventID: uuid(),
          eventName: 'INSERT',
          dynamodb: {
            // @ts-expect-error
            NewImage: marshalledProfile,
          },
        },
      ],
    };

    const pdid = testClient.pdid;
    const update = BasicPerson.constructFromObject({
      [PipedrivePersonFieldKeys.uuid]: testClient.uuid,
      [PipedrivePersonFieldKeys.pipedriveName]: testClient.pipedriveName,
    });

    await handler(testEvent, getLambdaContext(lambdaFunctionName));

    const instance = new PersonsApi();
    const updatePerson = instance.updatePerson;

    expect(updatePerson).toHaveBeenCalled();
    expect(updatePerson).toHaveBeenLastCalledWith(pdid, update);
  });
});