import { Marshaller } from '@aws/dynamodb-auto-marshaller';
import { AWS_REGION, lambdaConfig } from '@svc/config';
import { handler } from '@svc/handlers/outbound/syncNewProject';
import { buildLowLevelProjectDetails } from '@svc/lib/ddbRepos/projects-repo';
import { PipedriveDealFieldKeys } from '@improovy/types';
import uuid from '@svc/lib/uuid';
import getLambdaContext from '@tests/utils/getLambdaContext';
import TestDataGenerator from '@tests/utils/testDataGenerator';
import { DynamoDBStreamEvent } from 'aws-lambda';
import { DealsApi, BasicPerson } from 'pipedrive';


jest.mock('pipedrive', () => {
  const original = jest.requireActual('pipedrive');
  const updateDeal = jest.fn();
  return {
    __esModule: true,
    ...original,
    DealsApi: jest.fn().mockImplementation(() => {
      return { updateDeal };
    }),
  };
});

jest.mock('@svc/lib/ddbRepos/idempotency-repo', () => {
  return {
    __esModule: true,
    wasDDBRecordSeen: () => false,
    addSeenDDBRecord: () => Promise.resolve(),
  };
});

const marshaller = new Marshaller();

const testDataGenerator = new TestDataGenerator(false);

const lambdaFunctionName = `${lambdaConfig.functionNamePrefix}DDBSyncNewProject`;

describe('`syncNewDeal` Lambda function', () => {
   
  it('updates the pipedrive deal with the uuid', async () => {
    const testProject = testDataGenerator.getProject({ overrides: { pdid: '10347' }, fieldsToExclude: ['isTest'] });
    const lowLevelProject = buildLowLevelProjectDetails(testProject);
    const marshalledProject = marshaller.marshallItem(lowLevelProject);

    const testEvent: DynamoDBStreamEvent = {
      Records: [
        {
          eventID: uuid(),
          eventName: 'INSERT',
          dynamodb: {
            NewImage: marshalledProject as { [key: string]: AWSLambda.AttributeValue },
          },
        },
      ],
    };

    const pdid = testProject.pdid;
    const update = BasicPerson.constructFromObject({
      [PipedriveDealFieldKeys.uuid]: testProject.uuid,
    });

    await handler(testEvent, getLambdaContext(lambdaFunctionName));

    const instance = new DealsApi();
    const updateDeal = instance.updateDeal;

    expect(updateDeal).toHaveBeenCalled();
    expect(updateDeal).toHaveBeenLastCalledWith(pdid, update);
  });
});