import { DynamoDBClient } from '@aws-sdk/client-dynamodb';
// import { InvokeCommand, InvokeCommandInput, LambdaClient } from '@aws-sdk/client-lambda';
import { DynamoDBDocumentClient } from '@aws-sdk/lib-dynamodb';
// import { fromUtf8 } from '@aws-sdk/util-utf8-node';
// import { ApiClient } from 'pipedrive';
import { S3Client } from '@aws-sdk/client-s3'

const getEnvString = (key: string, required: boolean = false) => {
  const val = process.env[key];
  if (required && !val) {
  	throw new Error(`Missing required environment variable: ${key}`);
  }
  return val!;
};
  
// if SERVERLESS_STAGE exists, then the test suite is running in the serverless CICD pipeline
export const STAGE = getEnvString('SERVERLESS_STAGE', false) || getEnvString('STAGE', true);
export const SERVICE_NAME = getEnvString('SERVICE_NAME', true);
  
// export const AWS_ACCOUNT_ID = getEnvString('AWS_ACCOUNT_ID', true);

export const AWS_REGION = getEnvString('REGION', true);

// const lambdaClient = new LambdaClient({ region: AWS_REGION });
const functionNamePrefix = `${SERVICE_NAME}-${STAGE}-`;

export const lambdaConfig = {
  functionNamePrefix,
//   invokeSync: (funcName: string, eventBody: { [Key: string]: any } | string) => {
//     const body = typeof eventBody === 'string' ? eventBody : JSON.stringify(eventBody);
    
//     const invokeParams: InvokeCommandInput = {
//       FunctionName: `${functionNamePrefix}${funcName}`,
//       Payload: fromUtf8(JSON.stringify({ body })),
//     };

//     return lambdaClient.send(new InvokeCommand(invokeParams));
//   },
};

const ddbClient = new DynamoDBClient({ region: AWS_REGION });
// const CLIENTS_TABLE = getEnvString('DDB_TABLE_CLIENTS', true);
// const PROJECTS_TABLE = getEnvString('DDB_TABLE_PROJECTS', true);
// const CALLRAIL_TABLE = getEnvString('DDB_TABLE_CALLRAIL', true);
const ACTIVITIES_TABLE = getEnvString('DDB_TABLE_ACTIVITIES', true);
const IDEMPOTENCY_TABLE = getEnvString('DDB_TABLE_IDEMPOTENCY', true);
// const gaConversionsTable = getEnvString('DDB_TABLE_GA_CONVERSIONS', true);
export const transcribeRoleArn = getEnvString('IAM_ROLE_TRANSCRIBE_ARN', true);

// export const PIPEDRIVE_PASSWORD = getEnvString('PIPEDRIVE_PASSWORD', true);

// export const CALLRAIL_SIGNING_KEY = getEnvString('CALLRAIL_SIGNING_KEY', true)

// export const pipedriveAuthHeader = { Authorization: `Basic ${Buffer.from(`pipedrive:${PIPEDRIVE_PASSWORD}`).toString('base64')}` };

export const ddb = {
//   ddbClient,
  documentClient: DynamoDBDocumentClient.from(ddbClient),
//   clientsTable: CLIENTS_TABLE,
//   projectsTable: PROJECTS_TABLE,
//   callRailTable: CALLRAIL_TABLE,
  idempotencyTable: IDEMPOTENCY_TABLE,
  activitiesTable: ACTIVITIES_TABLE,
  // gaConversionsTable,
};

export const s3Config = {
  callRecordingsBucket: getEnvString('S3_BUCKET_CALL_RECORDINGS'),
  callTranscriptsBucket: getEnvString('S3_BUCKET_CALL_TRANSCRIPTS'),
  s3Client: new S3Client({
    region: AWS_REGION
  }),
}

// export const apiGatewayConfig = {
//   domainName: getEnvString('API_GW_DOMAIN'),
//   getBaseUrl: () => `https://${getEnvString('API_GW_DOMAIN')}.execute-api.${AWS_REGION}.amazonaws.com/${STAGE}`,
// };

// export const TEST_DEAL_PDID = 10347;
// export const TEST_PERSON_PDID = 9877;
// export const TEST_PERSON_UUID = 'TEST_PD_PERSON_UUID';
// export const TEST_ACTIVITY_PDID = 67802;

// const PIPEDRIVE_API_KEY = getEnvString('PIPEDRIVE_API_KEY', true);

// doing this here might not work if the config file is not loaded before an api call is made
// ApiClient.instance.authentications.api_key.apiKey = PIPEDRIVE_API_KEY;

// const GOOGLE_ADS_ACCOUNT_ID = getEnvString('GOOGLE_ADS_ACCOUNT_ID', true).replace(/-/g, '')
// const GOOGLE_ADS_API_KEY = getEnvString('GOOGLE_ADS_API_KEY', true)
// const GOOGLE_ADS_CLIENT_ID = getEnvString('GOOGLE_ADS_CLIENT_ID', true)
// const GOOGLE_ADS_CLIENT_SECRET = getEnvString('GOOGLE_ADS_CLIENT_SECRET', true)
// const GOOGLE_ADS_REFRESH_TOKEN = getEnvString('GOOGLE_ADS_REFRESH_TOKEN', true)

// if (STAGE !== "prod" && GOOGLE_ADS_ACCOUNT_ID === '6570200896') throw new Error("prod google ads account id detected outside of prod stage");

// // CAUTION CHANGING GOOGLE API STUFF, TESTS MIGHT NOT DETECT ERRORS

// const prodConversionActionIDs = {
//   BOOKED: 928127451,
//   QUOTE_RECEIVED: 927292490,
//   QUOTE_REQUESTED: 928119562,
// }

// const devConversionActionIDs = {
//   BOOKED: 933050542,
//   QUOTE_REQUESTED: 933088200,
//   QUOTE_RECEIVED: 933087117
// }

// const buildConversionActionResourceName = (id: number) => {
//   return `customers/${GOOGLE_ADS_ACCOUNT_ID}/conversionActions/${id}`
//   // return `customers/${6570200896}/conversionActions/${id}`
// }

// const buildConversionActionResourceNames = () => {
//   let actionIDS;
//   if (STAGE === 'prod') {
//   // if (true) {
//     actionIDS = prodConversionActionIDs
//   } else {
//     actionIDS = devConversionActionIDs
//   }

//   return {
//     bookedRN: buildConversionActionResourceName(actionIDS.BOOKED),
//     quoteReceivedRN: buildConversionActionResourceName(actionIDS.QUOTE_RECEIVED),
//     quoteRequestedRN: buildConversionActionResourceName(actionIDS.QUOTE_REQUESTED),
//   }
// } 

// const clientOptions = {
//   client_id: GOOGLE_ADS_CLIENT_ID,
//   client_secret: GOOGLE_ADS_CLIENT_SECRET,
//   developer_token: GOOGLE_ADS_API_KEY,
// }

// const customerOptions = {
//   customer_id: GOOGLE_ADS_ACCOUNT_ID,
//   // customer_id: 6570200896,
//   refresh_token: GOOGLE_ADS_REFRESH_TOKEN,
// }

// export const gaConversionsAPI = {
//   adsAccountID: GOOGLE_ADS_ACCOUNT_ID.replace(/-/g, ''),
//   // adsAccountID: '6570200896'.replace(/-/g, ''),
//   apiKey: GOOGLE_ADS_API_KEY,
//   credentialsID: GOOGLE_ADS_CLIENT_ID,
//   credentialsSecret: GOOGLE_ADS_CLIENT_SECRET,
//   refreshToken: GOOGLE_ADS_REFRESH_TOKEN,
//   ...buildConversionActionResourceNames()
// }