import { Context, DynamoDBStreamEvent } from 'aws-lambda';
import log from '@dazn/lambda-powertools-logger';
import { Marshaller } from '@aws/dynamodb-auto-marshaller';
import { addSeenCallSid, wasCallSidSeen } from '@svc/lib/ddbRepos/idempotency-repo';
import { LowLevelActivity } from '@improovy/types';
import downloadCallRecordingFromCallSid from '@svc/lib/downloadCallRecordingFromCallSid';

const marshaller = new Marshaller();

// todo: receive justcall calls via webhook, use that to save the call to S3 and trigger trancription, 
// once transcription and activity with respective callsid exist, update the activity body

export const handler = async (event: DynamoDBStreamEvent, ctx: Context) => {
  log.debug('Received event', { event });

  
  for (let record of event.Records) {
    const activity = marshaller.unmarshallItem(record.dynamodb!.NewImage!) as unknown as LowLevelActivity;
    
    const callSid = activity.callSid;

    if (!callSid) throw new Error("missing callSid");

    const callSidWasSeen = await wasCallSidSeen(callSid, ctx.functionName);

    if (callSidWasSeen) {
      log.debug('callSid already seen, skipping');
      continue;
    }

    if (activity.isTest) {
      log.debug('activity flagged as test, skipping')
      continue;
    }

    await downloadCallRecordingFromCallSid(callSid)

    await addSeenCallSid(callSid, ctx.functionName);
  }
  return;
};

