import { Context, S3Event } from 'aws-lambda';
import log from '@dazn/lambda-powertools-logger';
import { addSeenS3Record, wasS3RecordSeen } from '@svc/lib/ddbRepos/idempotency-repo';
import { StartCallAnalyticsJobCommand, TranscribeClient } from '@aws-sdk/client-transcribe';
import { AWS_REGION, s3Config, transcribeRoleArn } from '@svc/config';

const trancribeClient = new TranscribeClient({region: AWS_REGION})

export const handler = async (event: S3Event, ctx: Context) => {
  log.debug('Received event', { event });
  
  for (let record of event.Records) {
    const recordWasSeen = await wasS3RecordSeen(record, ctx.functionName);
    if (recordWasSeen) {
      log.debug('record was seen')
      continue;
    }

    const key = record.s3.object.key;
    
    const command = new StartCallAnalyticsJobCommand({
      CallAnalyticsJobName: key,
      DataAccessRoleArn: transcribeRoleArn,
      Media: {
        MediaFileUri: `s3://${s3Config.callRecordingsBucket}/${key}`,
      },
      OutputLocation: `s3://${s3Config.callTranscriptsBucket}`,
      ChannelDefinitions: [
        {
          ChannelId: 1,
          ParticipantRole: 'AGENT'
        },
        {
          ChannelId: 0,
          ParticipantRole: 'CUSTOMER'
        }
      ]
    })
    
    await trancribeClient.send(command)

    await addSeenS3Record(record, ctx.functionName);
  }
  return;
};