import { ChangeSource } from "./common";

export enum ClientIncomingContactMethod {
  WEBFORM = 'WEBFORM',
  PHONE = 'PHONE',
  EMAIL = 'EMAIL',
  INTEGRATION = 'INTEGRATION',
  UNKNOWN = 'UNKNOWN',
}

export enum MutableClientProperties {
  gclid = 'gclid',
  incomingContactMethod = 'incomingContactMethod',
  isDeleted = 'isDeleted',
  pdid = 'pdid',
  lastUpdatedField = 'lastUpdatedField',
  pipedriveName = 'pipedriveName',
  changeSource = 'changeSource'
}

export enum ImmutableClientProperties {
  uuid = 'uuid',
  createdAt = 'createdAt',
}

export type ClientProperties = ImmutableClientProperties | MutableClientProperties;

export interface LowLevelClientProfile {
  pk: string,
  sk: 'Profile',
  type: 'ClientProfile',
  gsi2_pk?: string,
  gsi2_sk?: string,
  gclid?: string,
  incomingContactMethod: ClientIncomingContactMethod,
  isDeleted: boolean,
  uuid: string,
  createdAt: number,
  pdid?: string,
  lastUpdatedField?: MutableClientProperties,
  pipedriveName: string,
  isTest?: boolean
  changeSource: ChangeSource,
}

export interface ClientInterface {
  gclid?: string,
  incomingContactMethod: ClientIncomingContactMethod,
  isDeleted?: boolean,
  uuid?: string,
  createdAt: number,
  pdid?: string,
  contactMethods: ClientContactMethod[],
  lastUpdatedField?: MutableClientProperties,
  pipedriveName: string,
  isTest?: boolean
  changeSource: ChangeSource,
}

export enum ContactType {
  PHONE = 'PHONE',
  EMAIL = 'EMAIL',
  THUMBTACK = 'THUMBTACK',
}
export interface ClientContactMethod {
  contactType: ContactType,
  contactAddress: string,
  isDeleted?: boolean
}

export interface LowLevelClientContactMethod {
  pk: string,
  sk: string,
  gsi1_pk: string,
  gsi1_sk: string,
  gsi2_pk?: string,
  gsi2_sk?: string,
  type: 'ClientContactMethod',
  contactType: ContactType,
  contactAddress: string,
  clientUuid: string,
  isDeleted?: boolean
}

export interface LowLevelClient {
  profile: LowLevelClientProfile,
  contactMethods: LowLevelClientContactMethod[],
}

export interface GenerateTestClientInput {
  fieldsToGenerate?: string[],
  fieldsToExclude?: string[],
  overrides?: Partial<ClientInterface>,
}