import { DeleteObjectCommand } from '@aws-sdk/client-s3';
import { GetCallAnalyticsJobCommand, GetCallAnalyticsJobCommandInput, TranscribeClient } from '@aws-sdk/client-transcribe';
import { AWS_REGION, lambdaConfig, s3Config } from '@svc/config';
import downloadCallRecordingFromCallSid from '@svc/lib/downloadCallRecordingFromCallSid';
import uuid from '@svc/lib/uuid';
import TestDataGenerator from '@tests/utils/testDataGenerator';

const testDataGenerator = new TestDataGenerator();

const lambdaFunctionName = `${lambdaConfig.functionNamePrefix}S3TriggerCallAnalytics`;

const awsTestOptions = {
  region: AWS_REGION,
  function: lambdaFunctionName,
};

const transcribeConfig = {
  region: AWS_REGION
}

const trancribeClient = new TranscribeClient(transcribeConfig)

describe('`triggerCallAnalytics` Lambda function', () => {
  const testCallSid = 'CA9e6b180e184feb32512eb3ceca3895a3';
  const testKey = `${uuid()}-testing`;
  it('is triggered when a new recording is added to the call recordings bucket', async () => {
    await downloadCallRecordingFromCallSid(testCallSid, testKey)

    await expect({...awsTestOptions, timeout: 50000}).toHaveLog(testKey)
  });

  it('starts a call analytics job referencing the recording', async () => {
    const input: GetCallAnalyticsJobCommandInput = {
      CallAnalyticsJobName: testKey
    }

    const job = await trancribeClient.send(new GetCallAnalyticsJobCommand(input))

    expect(job).toBeTruthy()
  });
  
  afterAll(async () => {
    await testDataGenerator.deleteAll()
    const bucketParams = { 
      Bucket: s3Config.callRecordingsBucket,
      Key: testKey
    }

    await s3Config.s3Client.send(new DeleteObjectCommand(bucketParams))
  });
});
